

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   DEVIL Screensaver System 1.05                                 *
    *   Copyright  2005 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2005-09-18   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


/*

	-- COMPILE INFO --
	libs:		MSVCRT.LIB kernel32.lib user32.lib gdi32.lib
	threads:	multi-threaded
	opts:		disable all default libs

*/



#include "Devil.h"
// #include "DevilConfig.h"



bool FACE_Live();
void FACE_Die();
LRESULT CALLBACK FACE_Master( HWND, UINT, WPARAM, LPARAM );
void FACE_Render( const double );
void FACE_Config( const HWND );
// bool FACE_Init3D();
// void FACE_Cleanup3D();
// bool FACE_SwitchRes( int *, int *, UINT *, UINT * );
// void FACE_RestoreRes();



// DEVIL_CONFIG g_CONFIG;

DEVIL_INTERFACE g_FACE =
{
	NULL,					// Live
	NULL,					// Die
	FACE_Master,			// Master
	FACE_Render,			// Render
	FACE_Config,			// Config
	NULL,					// Init3D
	NULL,					// Cleanup3D
	NULL,					// SwitchRes
	NULL,					// RestoreRes
	true,					// bEnableFrameLimiter
	25,						// uMaxFPS
	true,					// bMouseExit
	true,					// bAnyKeyExit

	NULL,					// hwnd
	DEVIL_MODE_UNKNOWN,		// Mode
	NULL,					// Slave
	0.0						// fFPS
};


UINT g_uClientWidth = 0;
UINT g_uClientHeight = 0;


#define USE_VIDEO_FOR_WINDOWS
#include "Pool.h"
#include "resource.h"

#ifdef USE_VIDEO_FOR_WINDOWS
# include <vfw.h>
#endif

Pool g_POOL;


/*
// ***************************************************************************************
bool FACE_Live()
{
	// YOUR CODE HERE

	return true;
}
*/

/*
// ***************************************************************************************
void FACE_Die()
{
	// YOUR CODE HERE
}
*/


// ***************************************************************************************
LRESULT CALLBACK FACE_Master( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
#ifdef USE_VIDEO_FOR_WINDOWS	
	static HDRAWDIB hdib = NULL;
#endif

	switch( message )
	{
	case WM_CREATE:
		{
			RECT r;
			if( GetClientRect( hwnd, &r ) )
			{
				g_uClientWidth = r.right - r.left;
				g_uClientHeight = r.bottom - r.top;
			}
			
#ifdef USE_VIDEO_FOR_WINDOWS
			// Open Video for Windows
			hdib = DrawDibOpen();
#endif
			break;
		}
	case WM_SIZE:
		g_uClientWidth  = LOWORD( lp );
		g_uClientHeight = HIWORD( lp );

		break;

	case WM_PAINT:
		{
#ifdef USE_VIDEO_FOR_WINDOWS
			HDC hdc = GetDC(hwnd);
			DrawDibDraw( hdib, hdc, 0, 0, g_uClientWidth, g_uClientHeight, &g_POOL.GetImageInfo()->bmiHeader, g_POOL.GetImage(), 0, 0, g_POOL.GetImageWidth(), g_POOL.GetImageHeight(), 0 );
			ReleaseDC( hwnd, hdc );
#else
			PAINTSTRUCT ps;
			HDC hdc = BeginPaint( hwnd, &ps );
			StretchDIBits( hdc, 0, 0, g_uClientWidth, g_uClientHeight, 0, 0, g_POOL.GetImageWidth(), g_POOL.GetImageHeight(), g_POOL.GetImage(), g_POOL.GetImageInfo(), DIB_RGB_COLORS, SRCCOPY );
			EndPaint( hwnd, &ps );
#endif
			break;
		}

#ifdef USE_VIDEO_FOR_WINDOWS
		case WM_DESTROY:
			// Close Video for Windows
			DrawDibClose( hdib );
			break;
#endif
	}

	return CallWindowProc( g_FACE.Slave, hwnd, message, wp, lp );
}



// ***************************************************************************************
void FACE_Render( const double fPassedTime )
{
	g_POOL.NextFrame();
	RedrawWindow( g_FACE.hwnd, NULL, NULL, RDW_INVALIDATE );
}



// ***************************************************************************************
BOOL CALLBACK WndprocConfig( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	if( WM_COMMAND == message )
	{
		if( IDCANCEL == LOWORD( wp ) )		// [Escape] or button
		{
			EndDialog( hwnd, 0 );
		}
	}
	return FALSE;
}



// ***************************************************************************************
void FACE_Config( const HWND hParent )
{
	DialogBox( GetModuleHandle( NULL ), MAKEINTRESOURCE( IDD_CONFIG ), hParent, WndprocConfig );
}


/*
// ***************************************************************************************
bool FACE_Init3D()
{
	// YOUR CODE HERE

	return true;
}
*/

/*
// ***************************************************************************************
void FACE_Cleanup3D()
{
	// YOUR CODE HERE
}
*/

/*
// ***************************************************************************************
bool FACE_SwitchRes( int * x, int * y, UINT * uWidth, UINT * uHeight )
{
	// YOUR CODE HERE

	return true;
}
*/

/*
// ***************************************************************************************
void FACE_RestoreRes()
{
	// YOUR CODE HERE
}
*/
