

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   DEVIL Screensaver System 1.05                                 *
    *   Copyright  2005 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2005-09-18   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


#include "DevilTimer.h"


// Initialize static members
bool	DEVIL_TIMER::bInited			= false;
bool	DEVIL_TIMER::bUseHardwareTimer	= false;
double	DEVIL_TIMER::fFactor			= 1.0;



// *****************************************************************************
DEVIL_TIMER::DEVIL_TIMER()
{
	if( !bInited )
	{
		choose_timer();
	}
	uLast = get_current_time();
}



// *****************************************************************************
void DEVIL_TIMER::choose_timer()
{
	LONGLONG iFrequency;
	bUseHardwareTimer = !!QueryPerformanceFrequency( ( LARGE_INTEGER * )&iFrequency );
	if( bUseHardwareTimer )
	{
		fFactor = 1.0 / iFrequency;
	}
	else
	{
		fFactor = 0.001;
	}
}



// *****************************************************************************
LONGLONG DEVIL_TIMER::get_current_time()
{
	LONGLONG uRet = 0;

	if( bUseHardwareTimer )
	{
		QueryPerformanceCounter( ( LARGE_INTEGER * )&uRet );
	}
	else
	{
	    uRet = GetTickCount(); //timeGetTime();
	}

	return uRet;
}



// *****************************************************************************
double DEVIL_TIMER::get_passed_time( bool bUpdateLast )
{
	LONGLONG uNow = get_current_time();
	double fTimePassed = ( uNow - uLast ) * ( fFactor * 1000 );

	if( bUpdateLast )
	{
		uLast = uNow;
	}	

	return fTimePassed;
}



// *****************************************************************************
bool DEVIL_TIMER::has_passed( double fMS, bool bUpdateLast, double * fTimePassed )
{
	LONGLONG uNow = get_current_time();
	double fTemp = ( uNow - uLast ) * ( fFactor * 1000 );

	if( fTemp >= fMS )
	{
		if( bUpdateLast )
		{
			uLast = uNow;
		}	

		if( fTimePassed != NULL )
		{
			*fTimePassed = fTemp;
		}

		return true;
	}
	else
	{
		return false;
	}
}
